import 'dart:typed_data';

import 'package:benchmark_harness/benchmark_harness.dart';
import 'package:chacha20/chacha20.dart';

void main() {
  print("Benchmarks:");
  print("--");
  new Chacha20BlockBenchmark().report();
  print("--");
  new Chacha20StreamBenchmark().report();
  print("--");
  new Chacha20NumerousSmallMessagesBenchmark().report();
  print("--");
}

class Chacha20BlockBenchmark extends BenchmarkBase {
  Uint32List state;
  Chacha20 chacha20;

  static const n = 1000000~/64;

  Chacha20BlockBenchmark() : super("1MB (block only)");

  @override
  void setup() {
    chacha20 = new Chacha20();
    chacha20.initialize(key: new Uint8List(32), nonce: null);
    state = new Uint32List(16);
  }

  @override
  void run() {
    for (var i = 0; i < n; i++) {
      chacha20.encryptState(state);
    }
  }
}

class Chacha20StreamBenchmark extends BenchmarkBase {
  Uint8List cleartext;
  Uint8List result;

  Chacha20StreamBenchmark() : super("1MB stream");

  @override
  void setup() {
    // 100 MB cleartext
    cleartext = new Uint8List(1 * 1000 * 1000);
    for (var i = 0; i < cleartext.lengthInBytes; i++) {
      cleartext[i] = 0xFF & i;
    }
    result = new Uint8List(cleartext.length);
  }

  @override
  void run() {
    final chacha20 = new Chacha20();
    chacha20.initialize(key: new Uint8List(32), nonce: null);
    chacha20.fillWithConverted(result, 0, cleartext, 0);
  }
}

class Chacha20NumerousSmallMessagesBenchmark extends BenchmarkBase {
  Uint8List cleartext;
  Uint8List result;

  Chacha20NumerousSmallMessagesBenchmark()
      : super("1MB in 10 000 small messages");

  @override
  void setup() {
    cleartext = new Uint8List(100);
    for (var i = 0; i < cleartext.lengthInBytes; i++) {
      cleartext[i] = 0xFF & i;
    }
    result = new Uint8List(cleartext.lengthInBytes);
  }

  @override
  void run() {
    for (var i = 0; i < 10000; i++) {
      final chacha20 = new Chacha20();
      chacha20.initialize(key: new Uint8List(32), nonce: null);
      chacha20.fillWithConverted(result, 0, cleartext, 0);
    }
  }
}